<?php

namespace Modules\DF\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\HTTP\Kernel;
use Modules\DF\Services\Lancamento as Service;
use Controllers\LogController;

class Lancamento{
    
    protected $service;
    protected $devolucao = false;
    protected $nflancada = false;
    protected $devolucaoLancada = false;
    
    protected $LogController;
    
    public function __construct(Service $service, $LogController) {
        $this->service = $service;
        $this->logController = $LogController;
    }
    
    public function entry(Request $request, Response $response){
        
        $start = date("H:i:s");
        if(empty($_POST)){
			$input = file_get_contents('php://input');
			$requestHTTP = json_decode($input, true); 
		}else{
			$requestHTTP = $_POST;
		}
		
        if(is_null($requestHTTP)){
            return $this->getResponse("E", ["Nenhuma nota enviada."], [], $response);
        }
        if(is_null($requestHTTP["Produtos"]) || empty($requestHTTP["Produtos"])){
                $returnResponse = $this->getResponse("E", ["Nenhum produto enviado."], [], $response);
        }
        
        $tax = $requestHTTP;
        $products = $tax["Produtos"];
        
        if(isset($tax["Encargos"])){
           $charges = $tax["Encargos"]; 
        }else{
           $charges = array(); 
        }
        if(isset($tax["DadosAdicionais"])){
            $dacs = $tax["DadosAdicionais"];
        }else{
            $dacs = array();
        }
        $primaryKeys = $this->service->getPrimaryKeys($tax);
        
            
        if(sizeof($primaryKeys["errors"]) > 0){
            return $returnResponse = $this->getResponse("E", $primaryKeys["errors"], [], $response);
        }else{
            $tax = $primaryKeys["taxUpdated"];
        }
        $this->logController->saveMasterLog();
        try{
            $validationEntry = $this->validateKeys($tax, $products, $charges, $dacs);
        }catch(\Exception $e){
            return $this->getResponse("E", [$e->getMessage()], [], $response);
        }
        
        if($validationEntry["SuccessValidation"] == true){
            $tax = $validationEntry["newTax"];
            $returnEntry = $this->entryTax($tax);
            if($returnEntry["SuccessEntry"] == true){
                $tax = $returnEntry["taxUpdated"];
                $returnCharges = $this->service->updateCharges($tax["Encargos"], $tax["CDFILIAL"], $tax["NRLANCTONF"]);
                if($returnCharges["SuccessCharge"] == true){
                    $productsSave = $this->service->saveProducts(
                                                                    $tax["NRAUTOFORN"],
                                                                    $tax["Produtos"], 
                                                                    $tax["CDEMPRESA"], 
                                                                    $tax["CDINSCESTA"], 
                                                                    $tax["CDTIPOOPER"], 
                                                                    $tax["CDFORNECED"], 
                                                                    $tax["CDFILIAL"], 
                                                                    $tax["NRLANCTONF"], 
                                                                    $tax["DTENTRSAID"], 
                                                                    $tax["CDOPERADOR"], 
                                                                    $tax["IDORIGDESTNF"], 
                                                                    $tax["IDENTRSAIDOP"],
                                                                    $tax["CDGRUPOFISCAL"], 
                                                                    $tax["IDINTEREST"], 
                                                                    $tax["CDFILIRECE"], 
                                                                    $tax["CDFILIEMIT"]);
                    $tax["Produtos"] = $productsSave["Updated"];
                    if($productsSave["SuccessProducts"] == true){
                        $anexos = [$tax["ANEXO"], $tax["CUSTOMCLOB1"]];
                        $returnAnexos = $this->service->saveAnexos($tax["CDFILIAL"], $tax["NRLANCTONF"], $anexos);
                        if($returnAnexos["SuccessAnexos"] == true){
                            $returnDacs = $this->service->saveDacs($tax["DadosAdicionais"], $tax["CDFILIAL"], $tax["NRLANCTONF"]);
                            if($returnDacs["SuccessDacs"] == true){
                                $returnTot = $this->service->insertNfTot($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                                if($returnTot["SuccessTot"] == true){
                                    $errors = $this->closeTax($tax);  
                                    if($errors["SuccessClose"] == true){
                                        $this->nflancada = true;
                                    }else{
                                        $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                        return $returnResponse = $this->getResponse("E", $errors["MessagesClose"], [], $response);
                                    }
                                }else{
                                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                    return $returnResponse = $this->getResponse("E", $errors["MessagesTot"], [], $response);
                                }
                            }else{
                                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                return $returnResponse = $this->getResponse("E", $returnDacs["MessagesDacs"], [],  $response);
                            }
                        }else{
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                            return $returnResponse = $this->getResponse("E", $returnAnexos["MessagesAnexos"], [],  $response);
                        }
                    }else{
                        if($productsSave["afBaixada"]){
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $productsSave["produtosBaixados"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                        }else{
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                        }
                        return $returnResponse = $this->getResponse("E", $productsSave["MessagesProducts"], [],  $response);
                    }
                }else{
                    if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                        $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                    return $returnResponse = $this->getResponse("E", $returnCharges["MessagesCharge"], [],  $response);
                }
            }else{
                $tax = $returnEntry["taxUpdated"];
                if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                return $returnResponse = $this->getResponse("E", $returnEntry["MessagesEntry"], [],  $response);
            }
        }else{
            if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
            return $this->getResponse("E", $validationEntry["messagesValidation"], [], $response);
        }
        $this->devolucao = $this->service->verificaDevolucao();
        if($this->devolucao){
            try{
                $messages = $this->service->insertDevolucao($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                if(sizeof($messages) > 0){
                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                    return $this->getResponse("E", $messages, [], $response);
                }
            }catch(\Exception $e){
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                return $this->getResponse("E", [$e->getMessage()], [], $response);
            }
        }
        if($this->nflancada && $tax["CDTIPOOPER"] == '01'){
            try{
                $this->service->taxApproved($tax["CDFILIAL"], $tax["NRLANCTONF"]);  
            }catch(\Exception $e){
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                return $this->getResponse("E", [$e->getMessage()], [], $response);
            }
        }
        $final = date("H:i:s");
        return $returnResponse = $this->getResponse("S", [], $returnEntry["KeysEntry"], $response);
    }
    
    public function validateKeys($tax, $products, $charges, $dacs){
        $returnService = $this->service->validateTax($tax, $products, $charges, $dacs);
        $messagesTax = $returnService["messages"];
        $tax = $returnService["taxAjusted"];
        
        $messagesTax = array_values($messagesTax);
        $messagesTax = !empty($messagesTax) ? $messagesTax : [];
        $messagesValidation = array_merge($messagesTax);
        
        $return = array(
            "SuccessValidation" => !(sizeof($messagesValidation) > 0),
            "messagesValidation"=> $messagesValidation,
            "newTax" => $tax
        );
        
        return $return;
    }
    
    public function entryTax($tax){
        $response = $this->service->saveTax($tax);
        $return = array(
            "SuccessEntry" => !(sizeof($response["messages"]) > 0),
            "MessagesEntry"=> $response["messages"],
            "KeysEntry" => $response["keys"],
            "taxUpdated" => $response["taxUpdated"]
        );
        
        return $return;
    }
    
    public function closeTax($tax){
        $response = $this->service->closeTax($tax);
        $return = array(
            "SuccessClose" => !(sizeof($response["messages"]) > 0),
            "MessagesClose"=> $response["messages"]
        );
        
        return $return;
    }
    
    public function getResponse($status, $messages, $keys, &$response){
        $retorno = array();
        $retorno["STATUS"] = $status;
        $retorno["MENSAGEM"] = $messages;
        $retorno["KEYS"] = $keys;
        header('Content-Type: application/json; charset=utf-8');
        $response->setMetadata($retorno);
    }
    
}
